#include <windows.h>
HINSTANCE LoadDLLFunctions(char *dllPathName);

// Define function pointer types for both RECALL and LEARNING DLLs
typedef int (*NSCreateNetwork)(void *&pNeuralNetwork, int networkType);
typedef int (*NSDestroyNetwork)(void *pNeuralNetwork);
typedef int (*NSGetInputOutputInfo)(void *pNeuralNetwork, int &numInputs, int &numOutputs);
typedef int (*NSGetResponse)(void *pNeuralNetwork, int exemplars, float *inputData, float *outputData);
typedef int (*NSGetSensitivity)(void *pNeuralNetwork, int exemplars, float *inputData, float *&sensitivityData, float dither);
typedef int (*NSLoadWeights)(void *pNeuralNetwork, const char *weightsPathName);
typedef int (*NSSaveWeights)(void *pNeuralNetwork, const char *weightsPathName);
typedef int (*NSRandomizeWeights)(void *pNeuralNetwork);
typedef int (*NSResetNetwork)(void *pNeuralNetwork);

// Define function pointer types for LEARNING DLLs only
typedef int (*NSTrain)(void *pNeuralNetwork, int epochs, int exemplars, float *inputData, float *desiredData, int cvExemplars, float *cvInputData, float *cvDesiredData);
typedef int (*NSGetBestCost)(void *pNeuralNetwork, float &bestCost);
typedef int (*NSSetBestCost)(void *pNeuralNetwork, float bestCost);
typedef int (*NSGetBestWeightsPathName)(void *pNeuralNetwork, char *bestWeightsPathName, int bufferLength, int &pathNameLength);
typedef int (*NSSetBestWeightsPathName)(void *pNeuralNetwork, const char *bestWeightsPathName);
typedef int (*NSGetCrossValidationEnabled)(void *pNeuralNetwork, bool &crossValidationEnabled);
typedef int (*NSSetCrossValidationEnabled)(void *pNeuralNetwork, bool crossValidationEnabled);
typedef int (*NSGetSaveBestWeightsEnabled)(void *pNeuralNetwork, bool &saveBestWeightsEnabled);
typedef int (*NSSetSaveBestWeightsEnabled)(void *pNeuralNetwork, bool saveBestWeightsEnabled);
typedef int (*NSGetSaveBestWeightsForTraining)(void *pNeuralNetwork, bool &saveBestWeightsForTraining);
typedef int (*NSSetSaveBestWeightsForTraining)(void *pNeuralNetwork, bool saveBestWeightsForTraining);
typedef int (*NSSetAutoComputeInputNormCoeff)(void *pNeuralNetwork, bool autoComputeInputNormCoeff);
typedef int (*NSGetAutoComputeInputNormCoeff)(void *pNeuralNetwork, bool &autoComputeInputNormCoeff);
typedef int (*NSSetAutoComputeOutputNormCoeff)(void *pNeuralNetwork, bool autoComputeOutputNormCoeff);
typedef int (*NSGetAutoComputeOutputNormCoeff)(void *pNeuralNetwork, bool &autoComputeOutputNormCoeff);
typedef int (*NSRemoveInputNormalization)(void *pNeuralNetwork);
typedef int (*NSRemoveOutputNormalization)(void *pNeuralNetwork);
typedef int (*NSSetInputNormMin)(void *pNeuralNetwork, float inputNormMin);
typedef int (*NSGetInputNormMin)(void *pNeuralNetwork, float &inputNormMin);
typedef int (*NSSetInputNormMax)(void *pNeuralNetwork, float inputNormMax);
typedef int (*NSGetInputNormMax)(void *pNeuralNetwork, float &inputNormMax);
typedef int (*NSSetOutputNormMin)(void *pNeuralNetwork, float outputNormMin);
typedef int (*NSGetOutputNormMin)(void *pNeuralNetwork, float &outputNormMin);
typedef int (*NSSetOutputNormMax)(void *pNeuralNetwork, float outputNormMax);
typedef int (*NSGetOutputNormMax)(void *pNeuralNetwork, float &outputNormMax);
typedef int (*NSSetNormalizeInputByChannel)(void *pNeuralNetwork, bool normalizeInputByChannel);
typedef int (*NSGetNormalizeInputByChannel)(void *pNeuralNetwork, bool &normalizeInputByChannel);
typedef int (*NSSetNormalizeOutputByChannel)(void *pNeuralNetwork, bool normalizeOutputByChannel);
typedef int (*NSGetNormalizeOutputByChannel)(void *pNeuralNetwork, bool &normalizeOutputByChannel);
typedef int (*NSGetCrossValidationCostData)(void *pNeuralNetwork, float *cvCostData);
typedef int (*NSGetCostData)(void *pNeuralNetwork, float *costData);
typedef int (*NSGetNumberOfEpochsTrained)(void *pNeuralNetwork, int &numberOfEpochsTrained);
typedef int (*NSGetEpochOfBestCost)(void *pNeuralNetwork, int &epochOfBestCost);
typedef int (*NSSeedRandom)(void *pNeuralNetwork, unsigned int seed);

// Declare function pointers for both RECALL and LEARNING DLLs
NSCreateNetwork createNetwork;
NSDestroyNetwork destroyNetwork;
NSLoadWeights loadWeights;
NSSaveWeights saveWeights;
NSRandomizeWeights randomizeWeights;
NSResetNetwork resetNetwork;
NSGetInputOutputInfo getInputOutputInfo;
NSGetResponse getResponse;
NSGetSensitivity getSensitivity;

// Declare function pointers for LEARNING DLLs only
NSGetBestCost getBestCost;
NSSetBestCost setBestCost;
NSTrain train;
NSGetBestWeightsPathName getBestWeightsPathName;
NSSetBestWeightsPathName setBestWeightsPathName;
NSGetCrossValidationEnabled getCrossValidationEnabled;
NSSetCrossValidationEnabled setCrossValidationEnabled;
NSGetSaveBestWeightsEnabled getSaveBestWeightsEnabled;
NSSetSaveBestWeightsEnabled setSaveBestWeightsEnabled;
NSGetSaveBestWeightsForTraining getSaveBestWeightsForTraining;
NSSetSaveBestWeightsForTraining setSaveBestWeightsForTraining;
NSSetAutoComputeInputNormCoeff setAutoComputeInputNormCoeff;
NSGetAutoComputeInputNormCoeff getAutoComputeInputNormCoeff;
NSSetAutoComputeOutputNormCoeff setAutoComputeOutputNormCoeff;
NSGetAutoComputeOutputNormCoeff getAutoComputeOutputNormCoeff;
NSRemoveInputNormalization removeInputNormalization;
NSRemoveOutputNormalization removeOutputNormalization;
NSSetInputNormMin setInputNormMin;
NSGetInputNormMin getInputNormMin;
NSSetInputNormMax setInputNormMax;
NSGetInputNormMax getInputNormMax;
NSSetOutputNormMin setOutputNormMin;
NSGetOutputNormMin getOutputNormMin;
NSSetOutputNormMax setOutputNormMax;
NSGetOutputNormMax getOutputNormMax;
NSSetNormalizeInputByChannel setNormalizeInputByChannel;
NSGetNormalizeInputByChannel getNormalizeInputByChannel;
NSSetNormalizeOutputByChannel setNormalizeOutputByChannel;
NSGetNormalizeOutputByChannel getNormalizeOutputByChannel;
NSGetCrossValidationCostData getCrossValidationCostData;
NSGetCostData getCostData;
NSGetNumberOfEpochsTrained getNumberOfEpochsTrained;
NSGetEpochOfBestCost getEpochOfBestCost;
NSSeedRandom seedRandom;

HINSTANCE LoadDLLFunctions(char *dllPathName)
{
	HINSTANCE hDLL = LoadLibrary(dllPathName);
	if (hDLL)
	{
		// Assign function pointers for both RECALL and LEARNING DLLs
		createNetwork = (NSCreateNetwork)GetProcAddress(hDLL, "createNetwork");
		destroyNetwork = (NSDestroyNetwork)GetProcAddress(hDLL, "destroyNetwork");
		getInputOutputInfo = (NSGetInputOutputInfo)GetProcAddress(hDLL, "getInputOutputInfo");
		getResponse = (NSGetResponse)GetProcAddress(hDLL, "getResponse");
		getSensitivity = (NSGetSensitivity)GetProcAddress(hDLL, "getSensitivity");
		loadWeights = (NSLoadWeights)GetProcAddress(hDLL, "loadWeights");
		saveWeights = (NSSaveWeights)GetProcAddress(hDLL, "saveWeights");
		randomizeWeights = (NSRandomizeWeights)GetProcAddress(hDLL, "randomizeWeights");
		resetNetwork = (NSResetNetwork)GetProcAddress(hDLL, "resetNetwork");

		// Assign function pointers for LEARNING DLLs only
		train = (NSTrain)GetProcAddress(hDLL, "train");
		getBestCost = (NSGetBestCost)GetProcAddress(hDLL, "getBestCost");
		setBestCost = (NSSetBestCost)GetProcAddress(hDLL, "setBestCost");
		getBestWeightsPathName = (NSGetBestWeightsPathName)GetProcAddress(hDLL, "getBestWeightsPathName");
		setBestWeightsPathName = (NSSetBestWeightsPathName)GetProcAddress(hDLL, "setBestWeightsPathName");
		getSaveBestWeightsEnabled = (NSGetSaveBestWeightsEnabled)GetProcAddress(hDLL, "getSaveBestWeightsEnabled");
		setSaveBestWeightsEnabled = (NSSetSaveBestWeightsEnabled)GetProcAddress(hDLL, "setSaveBestWeightsEnabled");
		getSaveBestWeightsForTraining = (NSGetSaveBestWeightsForTraining)GetProcAddress(hDLL, "getSaveBestWeightsForTraining");
		setSaveBestWeightsForTraining = (NSSetSaveBestWeightsForTraining)GetProcAddress(hDLL, "setSaveBestWeightsForTraining");
		getCrossValidationEnabled = (NSGetCrossValidationEnabled)GetProcAddress(hDLL, "getCrossValidationEnabled");
		setCrossValidationEnabled = (NSSetCrossValidationEnabled)GetProcAddress(hDLL, "setCrossValidationEnabled");
		setAutoComputeInputNormCoeff = (NSSetAutoComputeInputNormCoeff)GetProcAddress(hDLL, "setAutoComputeInputNormCoeff");
		getAutoComputeInputNormCoeff = (NSGetAutoComputeInputNormCoeff)GetProcAddress(hDLL, "getAutoComputeInputNormCoeff");
		setAutoComputeOutputNormCoeff = (NSSetAutoComputeOutputNormCoeff)GetProcAddress(hDLL, "setAutoComputeOutputNormCoeff");
		getAutoComputeOutputNormCoeff = (NSGetAutoComputeOutputNormCoeff)GetProcAddress(hDLL, "getAutoComputeOutputNormCoeff");
		removeInputNormalization = (NSRemoveInputNormalization)GetProcAddress(hDLL, "removeInputNormalization");
		removeOutputNormalization = (NSRemoveOutputNormalization)GetProcAddress(hDLL, "removeOutputNormalization");
		setInputNormMin = (NSSetInputNormMin)GetProcAddress(hDLL, "setInputNormMin");
		getInputNormMin = (NSGetInputNormMin)GetProcAddress(hDLL, "getInputNormMin");
		setInputNormMax = (NSSetInputNormMax)GetProcAddress(hDLL, "setInputNormMax");
		getInputNormMax = (NSGetInputNormMax)GetProcAddress(hDLL, "getInputNormMax");
		setOutputNormMin = (NSSetOutputNormMin)GetProcAddress(hDLL, "setOutputNormMin");
		getOutputNormMin = (NSGetOutputNormMin)GetProcAddress(hDLL, "getOutputNormMin");
		setOutputNormMax = (NSSetOutputNormMax)GetProcAddress(hDLL, "setOutputNormMax");
		getOutputNormMax = (NSGetOutputNormMax)GetProcAddress(hDLL, "getOutputNormMax");
		setNormalizeInputByChannel = (NSSetNormalizeInputByChannel)GetProcAddress(hDLL, "setNormalizeInputByChannel");
		getNormalizeInputByChannel = (NSGetNormalizeInputByChannel)GetProcAddress(hDLL, "getNormalizeInputByChannel");
		setNormalizeOutputByChannel = (NSSetNormalizeOutputByChannel)GetProcAddress(hDLL, "setNormalizeOutputByChannel");
		getNormalizeOutputByChannel = (NSGetNormalizeOutputByChannel)GetProcAddress(hDLL, "getNormalizeOutputByChannel");
		getCrossValidationCostData = (NSGetCrossValidationCostData)GetProcAddress(hDLL, "getCrossValidationCostData");
		getCostData = (NSGetCostData)GetProcAddress(hDLL, "getCostData");
		getNumberOfEpochsTrained = (NSGetNumberOfEpochsTrained)GetProcAddress(hDLL, "getNumberOfEpochsTrained");
		getEpochOfBestCost = (NSGetEpochOfBestCost)GetProcAddress(hDLL, "getEpochOfBestCost");
		seedRandom = (NSSeedRandom)GetProcAddress(hDLL, "seedRandom");

		if (!(
		// Check function pointers for both RECALL and LEARNING DLLs
		createNetwork &&
		destroyNetwork &&
		getInputOutputInfo &&
		getResponse &&
		getSensitivity &&
		loadWeights &&
		saveWeights &&
		randomizeWeights &&
		resetNetwork &&

		// Check function pointers for LEARNING DLLs only
		train &&
		getBestCost &&
		setBestCost &&
		getBestWeightsPathName &&
		setBestWeightsPathName &&
		getSaveBestWeightsEnabled &&
		setSaveBestWeightsEnabled &&
		getSaveBestWeightsForTraining &&
		setSaveBestWeightsForTraining &&
		getCrossValidationEnabled &&
		setCrossValidationEnabled &&
		setAutoComputeInputNormCoeff &&
		getAutoComputeInputNormCoeff &&
		setAutoComputeOutputNormCoeff &&
		getAutoComputeOutputNormCoeff &&
		removeInputNormalization &&
		removeOutputNormalization &&
		setInputNormMin &&
		getInputNormMin &&
		setInputNormMax &&
		getInputNormMax &&
		setOutputNormMin &&
		getOutputNormMin &&
		setOutputNormMax &&
		getOutputNormMax &&
		setNormalizeInputByChannel &&
		getNormalizeInputByChannel &&
		setNormalizeOutputByChannel &&
		getNormalizeOutputByChannel &&
		getCrossValidationCostData &&
		getCostData &&
		getNumberOfEpochsTrained &&
		getEpochOfBestCost &&
		seedRandom
		))
		{
			FreeLibrary(hDLL);
			hDLL = 0;
		}
	}
	return hDLL;
}
